@@ -3,10 +3,6 @@
 -- Copyright (C) 2021-2025 Rudy Matela
 -- Distributed under the 3-Clause BSD licence (see the file LICENSE).
 {-# LANGUAGE CPP, TemplateHaskell #-}
-#if __GLASGOW_HASKELL__ <= 710
-{-# LANGUAGE DeriveDataTypeable, StandaloneDeriving #-}
-import Data.Typeable
-#endif
 import Conjure
 import System.Environment (getArgs)
 
@@ -665,15 +661,7 @@ data Twitter  =  Tweet Int
               |  Empty
               deriving (Eq, Show)
 
-#if __GLASGOW_HASKELL__ <= 710
-deriving instance Typeable Twitter
-
-instance Express Twitter where
-  expr (Tweet n)  =  value "Tweet" Tweet :$ expr n
-  expr t  =  val t
-#else
 deriveExpress  ''Twitter
-#endif
 deriveListable ''Twitter
 deriveName     ''Twitter
 
