@@ -71,11 +71,12 @@ mem _ Leaf  =  False
 mem y (Node l x r)  =  y == x || (if y < x then mem y l else mem y r)
 
 insert :: Int -> Tree -> Tree
-insert x Leaf  =  unit x
-insert x (Node l y r)  =  case compare x y of
-  LT -> Node (insert x l) y r
-  EQ -> Node l y r
-  GT -> Node l y (insert x r)
+insert x Leaf          =  unit x  -- 2
+insert x (Node l y r)  =
+  case compare x y of             -- 6
+  LT -> Node (insert x l) y r     -- 12
+  EQ -> Node l y r                -- 16
+  GT -> Node l y (insert x r)     -- 22
 
 before :: Int -> Tree -> Tree
 before _ Leaf  =  Leaf
@@ -145,11 +146,9 @@ main = do
     , primOrdCaseFor (undefined :: Tree)
     ]
 
-  -- reachable in 15s after 32878 at size 14.
-  -- out of reach performance-wise (reaching 16 but need 19)
-  -- cf. the primOrdCaseFor version below
+  -- reachable in 15s, candidate #32878 at size 14.
   -- increase target to 50400 to reach...
-  conjureFromSpecWith args{target=10080} "before" beforeSpec
+  conjureFromSpecWith args{target=5040} "before" beforeSpec
     [ pr Leaf
     , prim "Node" Node
     , prim "==" ((==) :: Int -> Int -> Bool)
@@ -157,6 +156,16 @@ main = do
     , guard
     ]
 
+  -- reachable in 14s, candidate #32747 at size 14.
+  -- increase target to 50400 to reach...
+  conjureFromSpecWith args{target=5040} "beyond" beyondSpec
+    [ pr Leaf
+    , prim "Node" Node
+    , prim "==" ((==) :: Int -> Int -> Bool)
+    , prim "<=" ((<) :: Int -> Int -> Bool)
+    , guard
+    ]
+
   -- with 15, this reaches the solution, using 12 for shorter runtime
   -- using maxEquationSize = 7 reduces runtime from 13s to 11s
   conjureFromSpecWith args{maxSize = 12, maxEquationSize = 7} "before" beforeSpec
@@ -175,8 +184,9 @@ main = do
     , primOrdCaseFor (undefined :: Tree)
     ]
 
-  -- out of reach (reaching 12 but need 13)
-  conjureWith args{maxSize = 9} "union" union
+  -- reachable in 55s, candidate #173109 at size 13.
+  -- increase to 554400 to reach
+  conjureWith args{target=5544} "union" union
     [ pr Leaf
     , prim "Node" Node
     , prim "before" before
