@@ -44,6 +44,9 @@ module Conjure.Conjurable
   , conjureArgumentPats
   , conjureMostGeneralCanonicalVariation
   , conjureCasesFor
+  , conjurePatternsFor
+  , conjureArgumentPatterns
+  , conjurePatternsDebug
   )
 where
 
@@ -390,14 +393,10 @@ conjureTestBinds maxTests maxSearchTests nm f  =  take maxTests
 -- | Compute 'tiers' of values encoded as 'Expr's
 --   of the type of the given 'Expr'.
 conjureTiersFor :: Conjurable f => f -> Expr -> [[Expr]]
-conjureTiersFor f e  =  tf allTiers
-  where
-  allTiers :: [ [[Expr]] ]
-  allTiers  =  [etiers | (_,_,Just etiers,_,_,_) <- conjureReification f]
-  tf []  =  [[e]] -- no tiers found, keep variable
-  tf (etiers:etc)  =  case etiers of
-                      ((e':_):_) | typ e' == typ e -> etiers
-                      _                            -> tf etc
+conjureTiersFor f e  =
+  case [metiers | (eh,_,metiers,_,_,_) <- conjureReification f, typ e == typ eh] of
+  (Nothing:_) -> [[e]] -- no tiers found, keep variable
+  (Just etiers:_) -> etiers
 
 conjureGrounds :: Conjurable f => f -> Expr -> [Expr]
 conjureGrounds  =  grounds . conjureTiersFor
@@ -695,12 +694,12 @@ conjureWhatApplication what nm f  =  mostGeneralCanonicalVariation . foldApp
 
 -- | Computes tiers of sets of patterns for the given function.
 --
--- > > conjurePats [zero] "f" (undefined :: Int -> Int)
+-- > > conjurePats 1 [zero] "f" (undefined :: Int -> Int)
 -- > [[[f x :: Int]],[[f 0 :: Int,f x :: Int]]]
-conjurePats :: Conjurable f => [Expr] -> String -> f -> [[ [Expr] ]]
-conjurePats  es nm f  =  mapT (map mkApp)
-                      $  combinePatternOptions
-                      $  conjureArgumentPats es f
+conjurePats :: Conjurable f => Int -> [Expr] -> String -> f -> [[ [Expr] ]]
+conjurePats d es nm f  =  mapT (map mkApp)
+                       $  combinePatternOptions
+                       $  conjureArgumentPatterns d es f
   where
   mkApp  =  foldApp . (ef:)
          .  unfold
@@ -782,6 +781,82 @@ conjureCasesFor f eh  =
 --       non-top-level case breakdowns.
 --       It can be used in building a replacement for conjureArgumentPats
 
+
+-- | Towards a replacement for conjurePats that allows non-top-level breakdowns...
+--
+-- This function is currently experimental and unused in Conjure.
+--
+-- > > conjurePatternsFor (undefined :: [Int] -> ()) (is_)
+-- > [ [ [ _ ] ]
+-- > , [ [ [], (_:_) ] ]
+-- > , [ [ [], [_], (_:_:_) ] ]
+-- > , [ [ [], [_], [_,_], (_:_:_:_) ] ]
+-- > , ...
+-- > ]
+--
+-- > > let mis_ = hole (undefined :: [Maybe Int])
+-- > > conjurePatternsFor (undefined :: [Maybe Int] -> ()) mis_
+-- > [ [ [ _ ] ]
+-- > , [ [ [], (_:_) ]
+-- > , [ [ [], (Nothing:_), (Just _:_) ]
+-- >   , [ [], [_], (_:_:_) ] ]
+-- > , [ [ [], [Nothing], [Just _], (_:_:_) ] ]
+-- > , ...
+-- > ]
+--
+-- Ints are /crudely/ supported
+--
+-- > ghci> putLL 4 $ conjurePatternsDebug [zero] (undefined :: [Int])
+-- > [_ :: [Int]]
+-- >
+-- > [[] :: [Int],_:_ :: [Int]]
+-- >
+-- > [[] :: [Int],[_] :: [Int],_:_:_ :: [Int]]
+-- > [[] :: [Int],_:_ :: [Int],0:_ :: [Int]]
+conjurePatternsFor :: Conjurable f => Int -> [Expr] -> f -> Expr -> [[ [Expr] ]]
+conjurePatternsFor d es f  =  cpf d . holeAsTypeOf
+  where
+  casesFor  =  conjureCasesFor f
+  cpf :: Int -> Expr -> [[ [Expr] ]]
+  cpf d h
+    | d == 0     =  [[ [h] ]]
+    | null cs    =  mapT (++ [h]) $ setsOf [[e] | e <- es, typ e == typ h]
+    | otherwise  =  [ [h] ] : rest
+    where
+    cs  =  casesFor h
+    rest :: [[ [Expr] ]]
+    rest  =  mapT nubSort
+          $  mapT (concatMap $ unfold . fill (fold cs))
+          $  productsWith prods
+          $  map (cpf (d-1))
+          $  concatMap holes cs
+    -- TODO: avoid nubSorting here, by somehow not generating repeats
+    -- the repeats appear because of the way we fold before concatMapping
+    -- this may cause a significant performance issue on wider types ([Bool])
+
+  productsWith :: ([a] -> b) -> [ [[a]] ] -> [[b]]
+  productsWith f  =  mapT f . products
+  -- TODO: move productsWith to LeanCheck?
+
+
+-- | A drop-in replacement for conjureArgumentPats.
+--
+-- Still not in use by Conjure.
+conjureArgumentPatterns :: Conjurable f => Int -> [Expr] -> f -> [ [[ [Expr] ]] ]
+conjureArgumentPatterns depth es f  =  map (conjurePatternsFor depth es f) $ conjureArgumentHoles f
+
+
+-- | Use this instead of conjurePatternsFor for simpler calling.
+--
+-- This is an interface for running internal experiments.
+-- It will go away in a future version of Conjure.
+conjurePatternsDebug :: Conjurable a => [Expr] -> a -> [[ [Expr] ]]
+conjurePatternsDebug es a  =  conjurePatternsFor (-1) es foo (hole a)
+  where
+  foo x  =  ()  where _  =  x `asTypeOf` a
+
+
+
 -- -- -- other Conjurable instances -- -- --
 
 instance Conjurable Ordering where
