@@ -171,6 +171,7 @@ data Args = Args
   , maxDeconstructionSize :: Int  -- ^ maximum size of deconstructions (e.g.: @_ - 1@)
   , maxConstantSize       :: Int  -- ^ maximum size of constants (0 for no limit)
   , maxPatternSize        :: Int  -- ^ maximum size of patterns (0 for no limit)
+  , maxPatternDepth       :: Int  -- ^ maximum depth of patterns
 
   -- advanced & debug options --
   , carryOn               :: Bool -- ^ whether to carry on after finding a suitable candidate
@@ -214,10 +215,11 @@ args = Args
   , target                 =  10080
   , maxEvalRecursions      =  60
   , maxEquationSize        =   5
-  , maxSearchTests         =  100000
+  , maxSearchTests         =  110880
   , maxDeconstructionSize  =   4
   , maxConstantSize        =   0 -- unlimited
   , maxPatternSize         =   0 -- unlimited
+  , maxPatternDepth        =   1
 
   -- advanced & debug options --
   , carryOn                =  False
@@ -499,8 +501,8 @@ candidateDefnsC Args{..} nm f ps  =
   , deconstructions
   )
   where
-  pats | maxPatternSize > 0  =  take maxPatternSize $ conjurePats es nm f
-       | otherwise           =                        conjurePats es nm f
+  pats | maxPatternSize > 0  =  take maxPatternSize $ conjurePats maxPatternDepth es nm f
+       | otherwise           =                        conjurePats maxPatternDepth es nm f
   fss  =  concatMapT ps2fss pats
   -- replaces the any guard symbol with a guard of the correct type
   es  =  [if isGuardSymbol e then conjureGuard f else e | (e,_) <- ps]
@@ -613,28 +615,38 @@ candidateDefnsC Args{..} nm f ps  =
     where
     hs  =  nub $ conjureArgumentHoles f
 
+  recs :: Expr -> [[Expr]]
   recs ep  =  filterT (keepR ep)
            .  discardT (\e -> e == ep)
            $  recsV' (tail (vars ep))
+
+  recsV :: [Expr] -> [[Expr]]
   recsV vs  =  filterT (\e -> any (`elem` vs) (vars e))
             $  foldAppProducts ef [unguardT $ appsWith h vs | h <- conjureArgumentHoles f]
+
   -- like recs, but memoized
+  recs' :: Expr -> [[Expr]]
   recs' ep  =  fromMaybe errRP $ lookup ep eprs
     where
-    eprs = [(ep, recs ep) | ep <- possiblePats]
-  possiblePats  =  nubSort . concat . concat $ pats
+    eprs = [(ep, recs ep) | ep <- concat possiblePats]
+
+  possiblePats :: [[Expr]]
+  possiblePats  =  map (nubSort . concat) $ pats
+
   -- like recsV, but memoized
-  recsV' vs  =  fromMaybe errRV $ lookup vs evrs
+  recsV' :: [Expr] -> [[Expr]]
+  recsV' vs  =  fromMaybe errRV $ lookup (nubSort vs) evrs
     where
-    evrs = [(vs, recsV vs) | vs <- nubSort $ map (tail . vars) possiblePats]
+    evrs = [(vs, recsV vs) | vs <- concatMap nubSort $ mapT (nubSort . tail . vars) possiblePats]
+
+  errRP  =  error "candidateDefnsC: unexpected pattern.  You have found a bug, please report it."
+  errRV  =  error "candidateDefnsC: unexpected variables.  You have found a bug, please report it."
 
   thy  =  doubleCheck (===)
        .  theoryFromAtoms (===) maxEquationSize . (:[]) . nub
        $  cjHoles (prim nm f:ps) ++ [val False, val True] ++ es
   (===)  =  cjAreEqual (prim nm f:ps) maxTests
   isUnbreakable  =  conjureIsUnbreakable f
-  errRP  =  error "candidateDefnsC: unexpected pattern.  You have found a bug, please report it."
-  errRV  =  error "candidateDefnsC: unexpected variables.  You have found a bug, please report it."
 
 
 -- | Checks if the given pattern is a ground pattern.
