@@ -20,4 +20,8 @@ tests n  =
   , sets [1]  ==  [[1], [] :: [Int]]
   , sets [1,2]  ==  [[1,2], [1], [2], [] :: [Int]]
   , sets [1,2,3]  ==  [[1,2,3],[1,2],[1,3],[1],[2,3],[2],[3],[] :: [Int]]
+
+  , prods [ [1], [2], [3] ] == [[1,2,3]]
+  , prods [ [1,2], [3,4], [5] ] == [[1,3,5],[1,4,5],[2,3,5],[2,4,5]]
+  , prods [ [1,2], [3], [4,5] ] == [[1,3,4],[1,3,5],[2,3,4],[2,3,5]]
   ]
